@@ -13,33 +13,26 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #ifndef ASCIIEXPORT_H
 #define ASCIIEXPORT_H
 
-#include <qstring.h>
-#include <qcstring.h>
-#include <qfile.h>
-#include <qobject.h>
-
-#include <kdebug.h>
-#include <koFilter.h>
-#include <koTarStore.h>
-
-
-class ASCIIExport : public KoFilter {
+#include <KoFilter.h>
+//Added by qt3to4:
+#include <QByteArray>
 
+class ASCIIExport : public KoFilter
+{
     Q_OBJECT
 
 public:
-    ASCIIExport(KoFilter *parent, QString name);
+    ASCIIExport(QObject* parent, const QStringList&);
     virtual ~ASCIIExport() {}
-    /** filtering :) */
-    virtual const bool filter(const QCString &fileIn, const QCString &fileOut,
-                              const QCString& from, const QCString& to);
 
+    virtual KoFilter::ConversionStatus convert(const QByteArray& from, const QByteArray& to);
 };
+
 #endif // ASCIIEXPORT_H
