@@ -176,6 +176,24 @@ protected:
     KoOdfWriters *m_writers; // Needed to create new relationship for header/footer
 
     KoGenStyle m_currentTableCellStyle;
+    KoGenStyle m_currentTableCellStyleLeft;
+    KoGenStyle m_currentTableCellStyleRight;
+    KoGenStyle m_currentTableCellStyleTop;
+    KoGenStyle m_currentTableCellStyleBottom;
+    KoGenStyle m_currentTableCellStyleInsideV;
+    KoGenStyle m_currentTableCellStyleInsideH;
+
+    enum BorderSide {
+        TopBorder, BottomBorder, LeftBorder, RightBorder, InsideH, InsideV
+    };
+
+    //! Used for setting up properties for pages and paragraphs.
+    //! It is reversed map, so detecting duplicates is easy in applyBorders().
+    QMap<QString, BorderSide> m_borderStyles;
+
+    //! Used for setting up properties for pages and paragraphs.
+    //! It is reversed map, so detecting duplicates is easy in applyBorders().
+    QMap<QString, BorderSide> m_borderPaddings;
 
 private:
     void init();
@@ -184,18 +202,8 @@ private:
     //! Uses m_currentObjectWidthCm and m_currentObjectHeightCm for size (defaults to 2cm/2cm)
     void writeRect();
 
-    enum BorderSide {
-        TopBorder, BottomBorder, LeftBorder, RightBorder, InsideH, InsideV
-    };
-    //! Used for setting up properties for pages and paragraphs.
-    //! It is reversed map, so detecting duplicates is easy in applyBorders().
-    QMap<QString, BorderSide> m_borderStyles;
     QColor m_backgroundColor; //Documet background color
 
-    //! Used for setting up properties for pages and paragraphs.
-    //! It is reversed map, so detecting duplicates is easy in applyBorders().
-    QMap<QString, BorderSide> m_borderPaddings;
-
     //! Reads CT_Border complex type (p.392), used by children of pgBorders and children of pBdr
     KoFilter::ConversionStatus readBorderElement(BorderSide borderSide, const char *borderSideName);
 
