@@ -359,7 +359,14 @@ KoFilter::ConversionStatus DocxXmlStylesReader::read_style()
         }
         else if (type == "table") {
             m_currentTableStyle = KoGenStyle(KoGenStyle::TableStyle, odfType.toLatin1());
-            m_currentTableCellStyle = KoGenStyle(KoGenStyle::TableCellStyle, "table-cell");
+
+            // These are needed because ooxml stores table styles differently from odf
+            m_currentTableCellStyleLeft = KoGenStyle(KoGenStyle::TableCellStyle, "table-cell");
+            m_currentTableCellStyleRight = KoGenStyle(KoGenStyle::TableCellStyle, "table-cell");
+            m_currentTableCellStyleTop = KoGenStyle(KoGenStyle::TableCellStyle, "table-cell");
+            m_currentTableCellStyleBottom = KoGenStyle(KoGenStyle::TableCellStyle, "table-cell");
+            m_currentTableCellStyleInsideV = KoGenStyle(KoGenStyle::TableCellStyle, "table-cell");
+            m_currentTableCellStyleInsideH = KoGenStyle(KoGenStyle::TableCellStyle, "table-cell");
         }
     }
     MSOOXML::Utils::Setter<bool> currentTextStylePredefinedSetter(&m_currentTextStylePredefined, false);
@@ -434,49 +441,21 @@ KoFilter::ConversionStatus DocxXmlStylesReader::read_style()
         else if (type == "paragraph") {
             m_currentParagraphStyle.addAttribute("style:class", "text");
 
-            QBuffer buffer;
-            buffer.open(QIODevice::WriteOnly);
-            KoXmlWriter *tempWriter = new KoXmlWriter(&buffer);
-            m_currentTextStyle.writeStyleProperties(tempWriter, KoGenStyle::TextType);
-
-            QString content = QString::fromUtf8(buffer.buffer(), buffer.buffer().size());
-            delete tempWriter;
-
-            // We have to add the properties in a loop
-            // This works as long as text-properties don't have children
-            // Currenty KoGenStyle doesn't support adding this in other ways.             
-
-            int separatorLocation = content.indexOf(' ');
-            content = content.right(content.size() - (separatorLocation + 1));
-            separatorLocation = content.indexOf(' ');
-            if (separatorLocation < 0) {
-                separatorLocation = content.indexOf('/');
-            }
-            while (separatorLocation > 0) {
-                int equalSignLocation = content.indexOf('=');
-                if (equalSignLocation < 0) {
-                    break;
-                }
-                QString propertyName = content.left(equalSignLocation);
-                // Removing equal and one quota
-                content = content.right(content.size() - (equalSignLocation) - 2);
-                separatorLocation = content.indexOf('\"');
-                if (separatorLocation < 0) {
-                    separatorLocation = content.indexOf('/');
-                }
-                QString propertyValue = content.left(separatorLocation);
-                content = content.right(content.size() - (separatorLocation + 2));
-                m_currentParagraphStyle.addProperty(propertyName, propertyValue, KoGenStyle::TextType);
-            }
-
+            MSOOXML::Utils::copyPropertiesFromStyle(m_currentTextStyle, m_currentParagraphStyle, KoGenStyle::TextType);
             styleName = mainStyles->insert(m_currentParagraphStyle, styleName, insertionFlags);
-
         }
         else if (type == "table") {
             styleName = mainStyles->insert(m_currentTableStyle, styleName, insertionFlags);
-            if (!m_currentTableCellStyle.isEmpty()) {
-                mainStyles->insert(m_currentTableCellStyle, QString("%1-%2").arg(styleName).arg("cells"), insertionFlags);
-            }
+
+            // The names here are partly hard coded, it means in practise that if there was a style called
+            // 'style'-cellsLeft, then this logic would not work, however it is not very likely.
+            mainStyles->insert(m_currentTableCellStyleLeft, QString("%1-%2").arg(styleName).arg("cellsLeft"), insertionFlags);
+            mainStyles->insert(m_currentTableCellStyleRight, QString("%1-%2").arg(styleName).arg("cellsRight"), insertionFlags);
+            mainStyles->insert(m_currentTableCellStyleTop, QString("%1-%2").arg(styleName).arg("cellsTop"), insertionFlags);
+            mainStyles->insert(m_currentTableCellStyleBottom, QString("%1-%2").arg(styleName).arg("cellsBottom"), insertionFlags);
+            mainStyles->insert(m_currentTableCellStyleInsideV, QString("%1-%2").arg(styleName).arg("cellsInsideV"), insertionFlags);
+            mainStyles->insert(m_currentTableCellStyleInsideH, QString("%1-%2").arg(styleName).arg("cellsInsideH"), insertionFlags);
+
         }
         if (!nextStyleName.isEmpty()) {
             mainStyles->insertStyleRelation(styleName, nextStyleName, "style:next-style-name");
