@@ -27,14 +27,14 @@
 
 #include "KPrShapeAnimations.h"
 #include "KPrPlaceholderShape.h"
-#include "KPrAnimationDirector.h"
+#include "animations/KPrAnimationCache.h"
 #include "KPrPageSelectStrategyBase.h"
 #include "kdebug.h"
 
-KPrShapeManagerAnimationStrategy::KPrShapeManagerAnimationStrategy( KoShapeManager * shapeManager, KPrAnimationDirector * animationDirector,
+KPrShapeManagerAnimationStrategy::KPrShapeManagerAnimationStrategy( KoShapeManager *shapeManager, KPrAnimationCache* animationCache,
                                                                     KPrPageSelectStrategyBase * strategy )
 : KoShapeManagerPaintingStrategy( shapeManager )
-, m_animationDirector( animationDirector )
+, m_animationCache( animationCache )
 , m_strategy( strategy )
 {
 }
@@ -47,15 +47,15 @@ KPrShapeManagerAnimationStrategy::~KPrShapeManagerAnimationStrategy()
 void KPrShapeManagerAnimationStrategy::paint( KoShape * shape, QPainter &painter, const KoViewConverter &converter, bool forPrint )
 {
     if ( ! dynamic_cast<KPrPlaceholderShape *>( shape ) && m_strategy->page()->displayShape( shape ) ) {
-        if ( m_animationDirector->shapeShown( shape ) ) {
-
-            qreal zoom;
-            converter.zoom(&zoom, &zoom);
+        if ( m_animationCache->value(shape, "visibility", true).toBool() ) {
             painter.save();
-            QTransform animationTransform = m_animationDirector->shapeTransform(shape);
-            animationTransform.translate(animationTransform.dx() * (zoom - 1), animationTransform.dy() * zoom);
-            QTransform transform(painter.matrix() * shape->absoluteTransformation( &converter ));
-            transform *= animationTransform;
+            QTransform animationTransform = m_animationCache->value(shape, "transform", QTransform()).value<QTransform>();;
+            QTransform transform(painter.transform() * shape->absoluteTransformation(&converter));
+            if (animationTransform.isScaling()) {
+                transform = animationTransform * transform;
+            } else {
+                transform = transform * animationTransform;
+            }
 
             painter.setTransform(transform);
             // paint shape
@@ -67,4 +67,6 @@ void KPrShapeManagerAnimationStrategy::paint( KoShape * shape, QPainter &painter
 
 void KPrShapeManagerAnimationStrategy::adapt( KoShape * shape, QRectF & rect )
 {
+    Q_UNUSED(shape)
+    Q_UNUSED(rect)
 }
