@@ -59,7 +59,6 @@
 #include "OdfLoadingContext.h"
 #include "OdfSavingContext.h"
 #include "RowColumnFormat.h"
-#include "Selection.h"
 #include "ShapeApplicationData.h"
 #include "Sheet.h"
 #include "Style.h"
@@ -103,8 +102,8 @@ public:
     Private() : sheet(0), column(0), row(0) {}
 
     Sheet*  sheet;
-uint    column  : 17; // KS_colMax
-uint    row     : 17; // KS_rowMax
+    uint    column  : 17; // KS_colMax
+    uint    row     : 17; // KS_rowMax
 };
 
 
@@ -351,8 +350,10 @@ Style Cell::effectiveStyle() const
 {
     Style style = sheet()->cellStorage()->style(d->column, d->row);
     // use conditional formatting attributes
-    if (Style* conditialStyle = conditions().testConditions(*this, sheet()->map()->styleManager()))
-        style.merge(*conditialStyle);
+    const Style conditionalStyle = conditions().testConditions(*this);
+    if (!conditionalStyle.isEmpty()) {
+        style.merge(conditionalStyle);
+    }
     return style;
 }
 
@@ -947,14 +948,14 @@ QDomElement Cell::save(QDomDocument& doc, int xOffset, int yOffset, bool era)
 
     Conditions conditions = this->conditions();
     if (!conditions.isEmpty()) {
-        QDomElement conditionElement = conditions.saveConditions(doc);
+        QDomElement conditionElement = conditions.saveConditions(doc, sheet()->map()->converter());
         if (!conditionElement.isNull())
             cell.appendChild(conditionElement);
     }
 
     Validity validity = this->validity();
     if (!validity.isEmpty()) {
-        QDomElement validityElement = validity.saveXML(doc);
+        QDomElement validityElement = validity.saveXML(doc, sheet()->map()->converter());
         if (!validityElement.isNull())
             cell.appendChild(validityElement);
     }
@@ -1072,7 +1073,7 @@ QString Cell::saveOdfCellStyle(KoGenStyle &currentCellStyle, KoGenStyles &mainSt
     if (!conditions.isEmpty()) {
         // this has to be an automatic style
         currentCellStyle = KoGenStyle(KoGenStyle::TableCellAutoStyle, "table-cell");
-        conditions.saveOdfConditions(currentCellStyle);
+        conditions.saveOdfConditions(currentCellStyle, sheet()->map()->converter());
     }
     return style().saveOdf(currentCellStyle, mainStyles, d->sheet->map()->styleManager());
 }
@@ -1161,7 +1162,7 @@ bool Cell::saveOdf(KoXmlWriter& xmlwriter, KoGenStyles &mainStyles,
 
     Validity validity = Cell(sheet(), column, row).validity();
     if (!validity.isEmpty()) {
-        GenValidationStyle styleVal(&validity);
+        GenValidationStyle styleVal(&validity, sheet()->map()->converter());
         xmlwriter.addAttribute("table:validation-name", tableContext.valStyle.insert(styleVal));
     }
     if (isFormula()) {
@@ -1314,19 +1315,19 @@ void Cell::saveOdfValue(KoXmlWriter &xmlWriter)
     };
 }
 
-bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
+bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext, const Style& style)
 {
     //Search and load each paragraph of text. Each paragraph is separated by a line break.
-    loadOdfCellText(element, tableContext);
+    loadOdfCellText(element, tableContext, style);
 
     //
     // formula
     //
     bool isFormula = false;
     if (element.hasAttributeNS(KoXmlNS::table, "formula")) {
-        kDebug(36003) << "cell:" << name() << "formula :" << element.attributeNS(KoXmlNS::table, "formula", QString());
         isFormula = true;
         QString oasisFormula(element.attributeNS(KoXmlNS::table, "formula", QString()));
+        kDebug(36003) << "cell:" << name() << "formula :" << oasisFormula;
         // each spreadsheet application likes to safe formulas with a different namespace
         // prefix, so remove all of them
         QStringList prefixes = QStringList() << "oooc:" << "kspr:" << "of:" << "msoxl:";
@@ -1398,7 +1399,7 @@ bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
                     currency = Currency(element.attributeNS(KoXmlNS::office, "currency", QString()));
                 }
 
-                if( style().isEmpty() ) {
+                if( style.isEmpty() ) {
                     Style style;
                     style.setCurrency(currency);
                     setStyle(style);
@@ -1572,7 +1573,7 @@ bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
             setComment(comment);
     }
 
-    loadOdfObjects(element, tableContext.odfContext);
+    loadOdfObjects(element, tableContext);
 
     return true;
 }
@@ -1588,7 +1589,7 @@ KoXmlElement namedItemNSWithSpan(const KoXmlNode& node, const char* nsURI, const
             }
             if (n.localName() == "span" && n.namespaceURI() == nsURI) {
                 KoXmlElement e = KoXml::namedItemNS(n, nsURI, localName); // not recursive
-                if(!e.isNull()) {
+                if (!e.isNull()) {
                     return e;
                 }
             }
@@ -1611,14 +1612,14 @@ static bool findDrawElements(const KoXmlElement& parent)
     return false;
 }
 
-void Cell::loadOdfCellText(const KoXmlElement& parent, OdfLoadingContext& tableContext)
+void Cell::loadOdfCellText(const KoXmlElement& parent, OdfLoadingContext& tableContext, const Style& style)
 {
     //Search and load each paragraph of text. Each paragraph is separated by a line break
     KoXmlElement textParagraphElement;
     QString cellText;
 
     bool multipleTextParagraphsFound = false;
-    const bool hasRichText = KoTextLoader::containsRichText(parent);
+    bool hasRichText = KoTextLoader::containsRichText(parent);
 
     forEachElement(textParagraphElement , parent) {
         if (textParagraphElement.localName() == "p" &&
@@ -1639,6 +1640,7 @@ void Cell::loadOdfCellText(const KoXmlElement& parent, OdfLoadingContext& tableC
                     QString link = textA.attributeNS(KoXmlNS::xlink, "href", QString());
                     cellText = textA.text();
                     setUserInput(cellText);
+                    hasRichText = false;
                     // The value will be set later in loadOdf().
                     if ((!link.isEmpty()) && (link[0] == '#'))
                         link = link.remove(0, 1);
@@ -1657,7 +1659,7 @@ void Cell::loadOdfCellText(const KoXmlElement& parent, OdfLoadingContext& tableC
             // this is because they would currently be loaded twice, once by the KoTextLoader
             // and later properly by the cell itself
 
-            QTextCharFormat format = style().asCharFormat();
+            QTextCharFormat format = style.asCharFormat();
             sheet()->map()->textStyleManager()->defaultParagraphStyle()->characterStyle()->copyProperties(format);
 
             KoTextLoader loader(*tableContext.shapeContext);
@@ -1674,13 +1676,13 @@ void Cell::loadOdfCellText(const KoXmlElement& parent, OdfLoadingContext& tableC
 
     //Enable word wrapping if multiple lines of text have been found.
     if (multipleTextParagraphsFound) {
-        Style style;
-        style.setWrapText(true);
-        setStyle(style);
+        Style newStyle;
+        newStyle.setWrapText(true);
+        setStyle(newStyle);
     }
 }
 
-void Cell::loadOdfObjects(const KoXmlElement &parent, KoOdfLoadingContext& odfContext)
+void Cell::loadOdfObjects(const KoXmlElement &parent, OdfLoadingContext& tableContext)
 {
     // Register additional attributes, that identify shapes anchored in cells.
     // Their dimensions need adjustment after all rows are loaded,
@@ -1695,13 +1697,12 @@ void Cell::loadOdfObjects(const KoXmlElement &parent, KoOdfLoadingContext& odfCo
                 KoXmlNS::table, "end-y",
                 "table:end-y"));
 
-    KoShapeLoadingContext shapeContext(odfContext, d->sheet->resourceManager());
     KoXmlElement element;
     forEachElement(element, parent) {
         if (element.namespaceURI() != KoXmlNS::draw)
             continue;
 
-        KoShape* shape = KoShapeRegistry::instance()->createShapeFromOdf(element, shapeContext);
+        KoShape* shape = KoShapeRegistry::instance()->createShapeFromOdf(element, *tableContext.shapeContext);
         if (!shape) {
             kDebug(36003) << "Unable to load shape.";
             continue;
@@ -1709,6 +1710,19 @@ void Cell::loadOdfObjects(const KoXmlElement &parent, KoOdfLoadingContext& odfCo
 
         d->sheet->addShape(shape);
 
+        // The position is relative to the upper left sheet corner until now. Move it.
+        QPointF position = shape->position();
+        // Remember how far we're off from the top-left corner of this cell
+        double offsetX = position.x();
+        double offsetY = position.y();
+        for (int col = 1; col < column(); ++col)
+            position += QPointF(d->sheet->columnFormat(col)->width(), 0.0);
+        for (int row = 1; row < this->row(); ++row)
+            position += QPointF(0.0, d->sheet->rowFormat(row)->height());
+        shape->setPosition(position);
+
+        dynamic_cast<ShapeApplicationData*>(shape->applicationData())->setAnchoredToCell(true);
+
         // All three attributes are necessary for cell anchored shapes.
         // Otherwise, they are anchored in the sheet.
         if (!shape->hasAdditionalAttribute("table:end-cell-address") ||
@@ -1733,17 +1747,6 @@ void Cell::loadOdfObjects(const KoXmlElement &parent, KoOdfLoadingContext& odfCo
             continue;
         double endY = KoUnit::parseValue(string);
 
-        // The position is relative to the upper left sheet corner until now. Move it.
-        QPointF position = shape->position();
-        // Remember how far we're off from the top-left corner of this cell
-        double offsetX = position.x();
-        double offsetY = position.y();
-        for (int col = 1; col < column(); ++col)
-            position += QPointF(d->sheet->columnFormat(col)->width(), 0.0);
-        for (int row = 1; row < this->row(); ++row)
-            position += QPointF(0.0, d->sheet->rowFormat(row)->height());
-        shape->setPosition(position);
-
         // The column dimensions are already the final ones, but not the row dimensions.
         // The default height is used for the not yet loaded rows.
         // TODO Stefan: Honor non-default row heights later!
@@ -1756,7 +1759,6 @@ void Cell::loadOdfObjects(const KoXmlElement &parent, KoOdfLoadingContext& odfCo
             size += QSizeF(0.0, d->sheet->rowFormat(row)->height());
         shape->setSize(size);
 
-        dynamic_cast<ShapeApplicationData*>(shape->applicationData())->setAnchoredToCell(true);
     }
 }
 
@@ -1832,7 +1834,9 @@ bool Cell::load(const KoXmlElement & cell, int _xshift, int _yshift,
     KoXmlElement conditionsElement = cell.namedItem("condition").toElement();
     if (!conditionsElement.isNull()) {
         Conditions conditions;
-        conditions.loadConditions(sheet()->map()->styleManager(), conditionsElement);
+        Map *const map = sheet()->map();
+        ValueParser *const valueParser = map->parser();
+        conditions.loadConditions(conditionsElement, valueParser);
         if (!conditions.isEmpty())
             setConditions(conditions);
     } else if (paste && (mode == Paste::Normal || mode == Paste::NoBorder)) {
