@@ -1,5 +1,6 @@
 /* This file is part of the KDE project
  * Copyright (C) 2007 Fredy Yanardi <fyanardi@gmail.com>
+ * Copyright (C) 2010 Boudewijn Rempt <boud@kogmbh.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -25,7 +26,7 @@
 
 #include <KoToolManager.h>
 #include <KoShapeManager.h>
-#include <KoCanvasResourceProvider.h>
+#include <KoResourceManager.h>
 
 #include <klocale.h>
 #include <kdebug.h>
@@ -51,8 +52,8 @@ void KWStatisticsDocker::setView(KWView *view)
     QWidget *wdg = widget();
     if (wdg)
         delete wdg;
-    KWStatistics *statistics = new KWStatistics(view->kwcanvas()->resourceProvider(), view->kwdocument(),
-            view->kwcanvas()->shapeManager()->selection(), this);
+    KWStatistics *statistics = new KWStatistics(view->canvasBase()->resourceManager(), view->kwdocument(),
+            view->canvasBase()->shapeManager()->selection(), this);
     setWidget(statistics);
 }
 
@@ -66,7 +67,7 @@ QString KWStatisticsDockerFactory::id() const
     return QString("Statistics");
 }
 
-QDockWidget* KWStatisticsDockerFactory::createDockWidget()
+QDockWidget *KWStatisticsDockerFactory::createDockWidget()
 {
     KWStatisticsDocker *widget = new KWStatisticsDocker(m_view);
     widget->setObjectName(id());
@@ -74,5 +75,4 @@ QDockWidget* KWStatisticsDockerFactory::createDockWidget()
     return widget;
 }
 
-#include "KWStatisticsDocker.moc"
 
