@@ -1,6 +1,6 @@
 /* This file is part of the KDE project
  * Copyright (C) 2006, 2008 Thomas Zander <zander@kde.org>
- * Copyright (C) 2007 Boudewijn Rempt <boud@valdyas.org>
+ * Copyright (C) 2007-2010 Boudewijn Rempt <boud@valdyas.org>
  * Copyright (C) 2007-2008 Casper Boemann <cbr@boemann.dk>
  * Copyright (C) 2006-2007 Jan Hambrecht <jaham@gmx.net>
  * Copyright (C) 2009 Thorsten Zachmann <zachmann@kde.org>
@@ -25,12 +25,18 @@
 #define KOCANVASCONTROLLER_H
 
 #include "flake_export.h"
+#include <QObject>
 
-#include <QAbstractScrollArea>
+class QRect;
+class QRectF;
+class QPoint;
+class QPointF;
+class QSize;
 
 class KoShape;
 class KoCanvasBase;
 class KoView;
+class KoCanvasControllerProxyObject;
 
 /**
  * This widget is a wrapper around your canvas providing scrollbars.
@@ -50,12 +56,13 @@ class KoView;
  * The canvas widget is at most as big as the viewport of the scroll
  * area, and when the view on the document is near its edges, smaller.
  * In your canvas widget code, you can find the right place in your
- * document in pixel coordinates by adding the documentOffset
+ * document in view coordinates (pixels) by adding the documentOffset
  */
-class FLAKE_EXPORT KoCanvasController : public QAbstractScrollArea
+class FLAKE_EXPORT KoCanvasController
 {
-    Q_OBJECT
+
 public:
+
     /// An enum to alter the positioning and size of the canvas inside the canvas controller
     enum CanvasMode {
         AlignTop,     ///< canvas is top aligned if smaller than the viewport
@@ -65,16 +72,49 @@ public:
         Spreadsheet   ///< same as Infinite, but supports right-to-left layouts
     };
 
+
+    // proxy QObject
+    KoCanvasControllerProxyObject *proxyObject;
+
     /**
      * Constructor.
-     * @param parent the parent this widget will belong to
      */
-    explicit KoCanvasController(QWidget *parent = 0);
+    explicit KoCanvasController();
     virtual ~KoCanvasController();
+
+
+public:
+
     /**
-     * Reimplemented from QAbstractScrollArea.
+     * Returns the current margin that is used to pad the canvas with.
+     * This value is read from the KConfig property "canvasmargin"
      */
-    void scrollContentsBy(int dx, int dy);
+    virtual int margin() const;
+
+    /**
+     * Set the new margin to pad the canvas with.
+     */
+    virtual void setMargin(int margin);
+
+    /**
+     * Sets the how the canvas behaves if the zoomed document becomes smaller than the viewport.
+     * @param mode the new canvas mode, CanvasMode::Centered is the default value
+     */
+    virtual void setCanvasMode(KoCanvasController::CanvasMode mode);
+
+    /// Returns the current canvas mode
+    virtual KoCanvasController::CanvasMode canvasMode() const;
+
+
+    /**
+     * compatibility with QAbstractScrollArea
+     */
+    virtual void scrollContentsBy(int dx, int dy) = 0;
+
+    /**
+     * @return the size of the viewport
+     */
+    virtual QSize viewportSize() const = 0;
 
     /**
      * Set the shadow option -- by default the canvas controller draws
@@ -83,7 +123,7 @@ public:
      *
      * @param drawShadow if true, the shadow is drawn, if false, not
      */
-    void setDrawShadow(bool drawShadow);
+    virtual void setDrawShadow(bool drawShadow) = 0;
 
     /**
      * Set the new canvas to be shown as a child
@@ -92,67 +132,50 @@ public:
      * @param canvas the new canvas. The KoCanvasBase::canvas() will be called to retrieve the
      *        actual widget which will then be added as child of this one.
      */
-    void setCanvas(KoCanvasBase *canvas);
+    virtual void setCanvas(KoCanvasBase *canvas) = 0;
+
     /**
      * Return the currently set canvas
      * @return the currently set canvas
      */
-    KoCanvasBase *canvas() const;
-
-    /**
-     * Change the actual canvas widget used by the current canvas. This allows the canvas widget
-     * to be changed while keeping the current KoCanvasBase canvas and its associated resources as
-     * they are. This might be used, for example, to switch from a QWidget to a QGLWidget canvas.
-     * @param widget the new canvas widget.
-     */
-    void changeCanvasWidget(QWidget *widget);
+    virtual KoCanvasBase *canvas() const = 0;
 
     /**
      * return the amount of pixels vertically visible of the child canvas.
      * @return the amount of pixels vertically visible of the child canvas.
      */
-    int visibleHeight() const;
+    virtual int visibleHeight() const = 0;
+
     /**
      * return the amount of pixels horizontally visible of the child canvas.
      * @return the amount of pixels horizontally visible of the child canvas.
      */
-    int visibleWidth() const;
+    virtual int visibleWidth() const = 0;
+
     /**
      * return the amount of pixels that are not visible on the left side of the canvas.
      * The leftmost pixel that is shown is returned.
      */
-    int canvasOffsetX() const;
+    virtual int canvasOffsetX() const = 0;
+
     /**
      * return the amount of pixels that are not visible on the top side of the canvas.
      * The topmost pixel that is shown is returned.
      */
-    int canvasOffsetY() const;
+    virtual int canvasOffsetY() const = 0;
 
-    /**
-     * Sets the how the canvas behaves if the zoomed document becomes smaller than the viewport.
-     * @param mode the new canvas mode, CanvasMode::Centered is the default value
-     */
-    void setCanvasMode(CanvasMode mode);
-
-    /// Returns the current canvas mode
-    CanvasMode canvasMode() const;
-
-    /// Returns true if canvas uses QGLWidget (OpenGL can be used for painting on canvas)
-    bool isCanvasOpenGL() const;
-
-    /// Reimplemented from QObject
-    virtual bool eventFilter(QObject *watched, QEvent *event);
 
     /**
      * @brief Scrolls the content of the canvas so that the given rect is visible.
      *
-     * The rect is to be specified in document coordinates. The scrollbar positions
+     * The rect is to be specified in view coordinates (pixels). The scrollbar positions
      * are changed so that the centerpoint of the rectangle is centered if possible.
      *
      * @param rect the rectangle to make visible
      * @param smooth if true the viewport translation will make be just enough to ensure visibility, no more.
+     * @see KoViewConverter::documentToView()
      */
-    void ensureVisible(const QRectF &rect, bool smooth = false);
+    virtual void ensureVisible(const QRectF &rect, bool smooth = false) = 0;
 
     /**
      * @brief Scrolls the content of the canvas so that the given shape is visible.
@@ -161,59 +184,48 @@ public:
      *
      * @param shape the shape to make visible
      */
-    void ensureVisible(KoShape *shape);
-
-    /**
-     * will cause the toolOptionWidgetsChanged to be emitted and all
-     * listeners to be updated to the new widget.
-     *
-     * FIXME: This doesn't belong her and it does an
-     * inherits("KoView") so it too much tied to komain
-     *
-     * @param widgets the map of widgets
-     */
-    void setToolOptionWidgets(const QMap<QString, QWidget *> &widgets);
+    virtual void ensureVisible(KoShape *shape) = 0;
 
     /**
      * @brief zooms in around the center.
      *
-     * The center must be specified in document coordinates. The scrollbar positions
+     * The center must be specified in view coordinates (pixels). The scrollbar positions
      * are changed so that the center becomes center if possible.
      *
      * @param center the position to zoom in on
      */
-    void zoomIn(const QPoint &center);
+    virtual void zoomIn(const QPoint &center) = 0;
 
     /**
      * @brief zooms out around the center.
      *
-     * The center must be specified in pixel coordinates. The scrollbar positions
+     * The center must be specified in view coordinates (pixels). The scrollbar positions
      * are changed so that the center becomes center if possible.
      *
      * @param center the position to zoom out around
      */
-    void zoomOut(const QPoint &center);
+    virtual void zoomOut(const QPoint &center) = 0;
 
     /**
      * @brief zooms around the center.
      *
-     * The center must be specified in pixel coordinates. The scrollbar positions
+     * The center must be specified in view coordinates (pixels). The scrollbar positions
      * are changed so that the center becomes center if possible.
      *
      * @param center the position to zoom around
      * @param zoom the zoom to apply
      */
-    void zoomBy(const QPoint &center, qreal zoom);
+    virtual void zoomBy(const QPoint &center, qreal zoom) = 0;
 
     /**
      * @brief zoom so that rect is exactly visible (as close as possible)
      *
-     * The rect must be specified in pixel coordinates. The scrollbar positions
+     * The rect must be specified in view coordinates (pixels). The scrollbar positions
      * are changed so that the center of the rect becomes center if possible.
      *
-     * @param rect the rect in pixel coords that should fit the view afterwards
+     * @param rect the rect in view coordinates (pixels) that should fit the view afterwards
      */
-    void zoomTo(const QRect &rect);
+    virtual void zoomTo(const QRect &rect) = 0;
 
     /**
      * @brief repositions the scrollbars so previous center is once again center
@@ -224,52 +236,92 @@ public:
      *
      * The success of this method is limited by the size of thing. But we try our best.
      */
-    void recenterPreferred();
+    virtual void recenterPreferred() = 0;
 
     /**
-     * Sets the preferred center point in pixels.
+     * Sets the preferred center point in view coordinates (pixels).
      * @param viewPoint the new preferred center
      */
-    void setPreferredCenter(const QPoint &viewPoint);
+    virtual void setPreferredCenter(const QPoint &viewPoint) = 0;
 
-    /// Returns the currently set preferred center point in pixels
-    QPoint preferredCenter() const;
+    /// Returns the currently set preferred center point in view coordinates (pixels)
+    virtual QPoint preferredCenter() const = 0;
 
     /**
      * Move the canvas over the x and y distance of the parameter distance
-     * @param distance the distance in pixels.  A positive distance means moving the canvas up/left.
+     * @param distance the distance in view coordinates (pixels).  A positive distance means moving the canvas up/left.
      */
-    void pan(const QPoint &distance);
-
-    /**
-     * Returns the current margin that is used to pad the canvas with.
-     * This value is read from the KConfig property "canvasmargin"
-     */
-    int margin() const;
-
-    /**
-     * Set the new margin to pad the canvas with.
-     */
-    void setMargin(int margin);
+    virtual void pan(const QPoint &distance) = 0;
 
     /**
      * Get the position of the scrollbar
      */
-    QPoint scrollBarValue() const;
+    virtual QPoint scrollBarValue() const = 0;
 
     /**
      * Set the position of the scrollbar
      * @param value the new values of the scroll bars
      */
-    void setScrollBarValue(const QPoint &value);
+    virtual void setScrollBarValue(const QPoint &value) = 0;
 
     /**
-     * \internal
+     * Called when the size of your document in view coordinates (pixels) changes, for instance when zooming.
+     *
+     * @param newSize the new size, in view coordinates (pixels), of the document.
+     * @param recalculateCenter if true the offset in the document we center on after calling
+     *      recenterPreferred() will be recalculated for the new document size so the visual offset stays the same.
      */
+    virtual void updateDocumentSize(const QSize &sz, bool recalculateCenter) = 0;
+
+protected:
+
+    void setDocumentSize(const QSize &sz);
+    QSize documentSize() const;
+
+    void setPreferredCenterFractionX(qreal);
+    qreal preferredCenterFractionX() const;
+
+    void setPreferredCenterFractionY(qreal);
+    qreal preferredCenterFractionY() const;
+
+    void setDocumentOffset( QPoint &offset);
+    QPoint documentOffset() const;
+
+private:
+
     class Private;
-    KoCanvasController::Private *priv();
+    Private * const d;
+};
+
+
+class FLAKE_EXPORT KoCanvasControllerProxyObject : public QObject {
+
+    Q_OBJECT
+    Q_DISABLE_COPY(KoCanvasControllerProxyObject);
+
+public:
+
+    KoCanvasControllerProxyObject(KoCanvasController *canvasController, QObject *parent = 0);
+
+public:
+
+    // Convenience methods to invoke the signals from subclasses
+
+    void emitCanvasRemoved(KoCanvasController *canvasController) { emit canvasRemoved(canvasController); }
+    void emitCanvasSet(KoCanvasController *canvasController) { emit canvasSet(canvasController); }
+    void emitCanvasOffsetXChanged(int offset) { emit canvasOffsetXChanged(offset); }
+    void emitCanvasOffsetYChanged(int offset) { emit canvasOffsetYChanged(offset); }
+    void emitCanvasMousePositionChanged(const QPoint &position) { emit canvasMousePositionChanged(position); }
+    void emitDocumentMousePositionChanged(const QPointF &position) { emit documentMousePositionChanged(position); }
+    void emitSizeChanged(const QSize &size) { emit sizeChanged(size); }
+    void emitMoveDocumentOffset(const QPoint &point) { emit moveDocumentOffset(point); }
+    void emitZoomBy(const qreal factor) { emit zoomBy(factor); }
+
+    // Convenience method to retrieve the canvas controller for who needs to use QPointer
+    KoCanvasController *canvasController() const { return m_canvasController; }
 
 signals:
+
     /**
      * Emitted when a previously added canvas is about to be removed.
      * @param canvasController this object
@@ -296,13 +348,16 @@ signals:
 
     /**
      * Emitted when the cursor is moved over the canvas widget.
-     * @param position the position in widget pixels.
+     * @param position the position in view coordinates (pixels).
      */
     void canvasMousePositionChanged(const QPoint &position);
 
     /**
      * Emitted when the cursor is moved over the canvas widget.
      * @param position the position in document coordinates.
+     *
+     * Use \ref canvasMousePositionChanged to get the position
+     * in view coordinates.
      */
     void documentMousePositionChanged(const QPointF &position);
 
@@ -312,16 +367,6 @@ signals:
      */
     void sizeChanged(const QSize &size);
 
-    /**
-     * Emit the new tool option widgets to be used with this canvas.
-     */
-    void toolOptionWidgetsChanged(const QMap<QString, QWidget *> &map, QWidget *widgets);
-
-    /**
-     * Emit the new tool option widgets to be used with this canvas.
-     */
-    void toolOptionWidgetsChanged(const QMap<QString, QWidget *> &widgets);
-
     /**
      * Emitted whenever the document is scrolled.
      *
@@ -342,46 +387,17 @@ signals:
 public slots:
 
     /**
-     * Call this slot whenever the size of your document in pixels
+     * Call this slot whenever the size of your document in view coordinates (pixels)
      * changes, for instance when zooming.
-     * @param newSize the new size, in pixels, of the document.
+     * @param newSize the new size, in view coordinates (pixels), of the document.
      * @param recalculateCenter if true the offset in the document we center on after calling
      *      recenterPreferred() will be recalculated for the new document size so the visual offset stays the same.
      */
-    void setDocumentSize(const QSize &newSize, bool recalculateCenter = true);
-
-protected slots:
-
-    /// Called by the horizontal scrollbar when its value changes
-    void updateCanvasOffsetX();
-
-    /// Called by the vertical scrollbar when its value changes
-    void updateCanvasOffsetY();
-
-protected:
-    /// reimplemented from QWidget
-    virtual void paintEvent(QPaintEvent *event);
-    /// reimplemented from QWidget
-    virtual void resizeEvent(QResizeEvent *resizeEvent);
-    /// reimplemented from QWidget
-    virtual void dragEnterEvent(QDragEnterEvent *event);
-    /// reimplemented from QWidget
-    virtual void dropEvent(QDropEvent *event);
-    /// reimplemented from QWidget
-    virtual void dragMoveEvent(QDragMoveEvent *event);
-    /// reimplemented from QWidget
-    virtual void dragLeaveEvent(QDragLeaveEvent *event);
-    /// reimplemented from QWidget
-    virtual void wheelEvent(QWheelEvent *event);
-    /// reimplemented from QWidget
-    virtual void keyPressEvent(QKeyEvent *event);
-    /// reimplemented from QWidget
-    virtual bool focusNextPrevChild(bool next);
+    void updateDocumentSize(const QSize &newSize, bool recalculateCenter = true);
 
 private:
-    Q_PRIVATE_SLOT(d, void activate())
 
-    Private * const d;
+    KoCanvasController *m_canvasController;
 };
 
 #endif
