@@ -1,7 +1,7 @@
 /* This file is part of the KDE project
  * Copyright (c) 2006 Boudewijn Rempt (boud@valdyas.org)
- * Copyright (C) 2006-2007 Thomas Zander <zander@kde.org>
- * Copyright (C) 2006 Thorsten Zachmann <zachmann@kde.org>
+ * Copyright (C) 2006-2007, 2010 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2006,2008-2010 Thorsten Zachmann <zachmann@kde.org>
  * Copyright (C) 2007 Jan Hambrecht <jaham@gmx.net>
  *
  * This library is free software; you can redistribute it and/or
@@ -23,6 +23,7 @@
 #include "KoPathShapeFactory.h"
 #include "KoConnectionShapeFactory.h"
 #include "KoShapeLoadingContext.h"
+#include "KoTextOnShapeContainer.h"
 #include "KoShapeSavingContext.h"
 #include "KoShapeGroup.h"
 
@@ -43,9 +44,14 @@
 class KoShapeRegistry::Private
 {
 public:
+    void insertFactory(KoShapeFactoryBase *factory);
+    void init(KoShapeRegistry *q);
+
+    KoShape *createShapeInternal(const KoXmlElement &fullElement, KoShapeLoadingContext &context, const KoXmlElement &element) const;
+
 
     // Map namespace,tagname to priority:factory
-    QHash<QPair<QString, QString>, QMultiMap<int, KoShapeFactory*> > factoryMap;
+    QHash<QPair<QString, QString>, QMultiMap<int, KoShapeFactoryBase*> > factoryMap;
 };
 
 KoShapeRegistry::KoShapeRegistry()
@@ -58,7 +64,7 @@ KoShapeRegistry::~KoShapeRegistry()
     delete d;
 }
 
-void KoShapeRegistry::init()
+void KoShapeRegistry::Private::init(KoShapeRegistry *q)
 {
     KoPluginLoader::PluginsConfig config;
     config.whiteList = "FlakePlugins";
@@ -74,14 +80,14 @@ void KoShapeRegistry::init()
                                      config);
 
     // Also add our hard-coded basic shape
-    add(new KoPathShapeFactory(this, QStringList()));
-    add(new KoConnectionShapeFactory(this));
+    q->add(new KoPathShapeFactory(q, QStringList()));
+    q->add(new KoConnectionShapeFactory(q));
 
     // Now all shape factories are registered with us, determine their
     // assocated odf tagname & priority and prepare ourselves for
     // loading ODF.
 
-    QList<KoShapeFactory*> factories = values();
+    QList<KoShapeFactoryBase*> factories = q->values();
     for (int i = 0; i < factories.size(); ++i) {
         insertFactory(factories[i]);
     }
@@ -91,34 +97,39 @@ KoShapeRegistry* KoShapeRegistry::instance()
 {
     K_GLOBAL_STATIC(KoShapeRegistry, s_instance)
     if (!s_instance.exists()) {
-        s_instance->init();
+        s_instance->d->init(s_instance);
     }
     return s_instance;
 }
 
-void KoShapeRegistry::addFactory(KoShapeFactory * factory)
+void KoShapeRegistry::addFactory(KoShapeFactoryBase * factory)
 {
     add(factory);
-    insertFactory(factory);
+    d->insertFactory(factory);
 }
 
-void KoShapeRegistry::insertFactory(KoShapeFactory * factory)
+void KoShapeRegistry::Private::insertFactory(KoShapeFactoryBase *factory)
 {
-    if (factory->odfNameSpace().isEmpty() || factory->odfElementNames().isEmpty()) {
+    const QList<QPair<QString, QStringList> > odfElements(factory->odfElements());
+
+    if (odfElements.isEmpty()) {
         kDebug(30006) << "Shape factory" << factory->id() << " does not have OdfNamespace defined, ignoring";
-    } else {
-        foreach(const QString & elementName, factory->odfElementNames()) {
+    }
+    else {
+        int priority = factory->loadingPriority();
+        for (QList<QPair<QString, QStringList> >::const_iterator it(odfElements.begin()); it != odfElements.end(); ++it) {
+            foreach (const QString &elementName, (*it).second) {
+                QPair<QString, QString> p((*it).first, elementName);
 
-            QPair<QString, QString> p(factory->odfNameSpace(), elementName);
+                QMultiMap<int, KoShapeFactoryBase*> & priorityMap = factoryMap[p];
 
-            QMultiMap<int, KoShapeFactory*> priorityMap = d->factoryMap[p];
+                priorityMap.insert(priority, factory);
 
-            d->factoryMap[p].insert(factory->loadingPriority(), factory);
-
-            kDebug(30006) << "Inserting factory" << factory->id() << " for"
-            << p << " with priority "
-            << factory->loadingPriority() << " into factoryMap making "
-            << d->factoryMap[p].size() << " entries. " << endl;
+                kDebug(30006) << "Inserting factory" << factory->id() << " for"
+                    << p << " with priority "
+                    << priority << " into factoryMap making "
+                    << priorityMap.size() << " entries. ";
+            }
         }
     }
 }
@@ -131,18 +142,36 @@ KoShape * KoShapeRegistry::createShapeFromOdf(const KoXmlElement & e, KoShapeLoa
     // If the element is in a frame, the frame is already added by the
     // application and we only want to create a shape from the
     // embedded element. The very first shape we create is accepted.
-    // XXX: we might want to have some code to determine which is the
-    // "best" of the creatable shapes.
+    //
+    // FIXME: we might want to have some code to determine which is
+    //        the "best" of the creatable shapes.
     if (e.tagName() == "frame" && e.namespaceURI() == KoXmlNS::draw) {
-
         KoXmlElement element;
         forEachElement(element, e) {
-            shape = createShapeInternal(e, context, element);
+            // Check for draw:object
+            if (element.tagName() == "object" && element.namespaceURI() == KoXmlNS::draw && element.hasChildNodes()) {
+                // Loop through the elements and find the first one
+                // that is handled by any shape.
+                KoXmlNode n = element.firstChild();
+                for (; !n.isNull(); n = n.nextSibling()) {
+                    if (n.isElement()) {
+                        shape = d->createShapeInternal(e, context, n.toElement());
+                        break;
+                    }
+                }
+            }
+            else {
+                // If not draw:object, e.g draw:image or draw:plugin
+                shape = d->createShapeInternal(e, context, element);
+            }
+
+            // If we found a shape that can handle the element in question, then break.
             if (shape) {
                 break;
             }
         }
     }
+
     // Hardwire the group shape into the loading as it should not appear
     // in the shape selector
     else if (e.localName() == "g" && e.namespaceURI() == KoXmlNS::draw) {
@@ -159,7 +188,7 @@ KoShape * KoShapeRegistry::createShapeFromOdf(const KoXmlElement & e, KoShapeLoa
             delete group;
         }
     } else {
-        shape = createShapeInternal(e, context, e);
+        shape = d->createShapeInternal(e, context, e);
     }
 
     if (shape) {
@@ -169,23 +198,42 @@ KoShape * KoShapeRegistry::createShapeFromOdf(const KoXmlElement & e, KoShapeLoa
     return shape;
 }
 
-KoShape * KoShapeRegistry::createShapeInternal(const KoXmlElement & fullElement, KoShapeLoadingContext & context, const KoXmlElement & element) const
+KoShape *KoShapeRegistry::Private::createShapeInternal(const KoXmlElement &fullElement,
+                                                       KoShapeLoadingContext &context,
+                                                       const KoXmlElement &element) const
 {
+    // Pair of namespace, tagname
     QPair<QString, QString> p = QPair<QString, QString>(element.namespaceURI(), element.tagName());
-    kDebug(30006) << p;
 
-    if (!d->factoryMap.contains(p)) return 0;
+    // Remove duplicate lookup.
+    if (!factoryMap.contains(p))
+        return 0; 
 
-    QMultiMap<int, KoShapeFactory*> priorityMap = d->factoryMap[p];
-    QList<KoShapeFactory*> factories = priorityMap.values();
+    QMultiMap<int, KoShapeFactoryBase*> priorityMap = factoryMap.value(p);
+    QList<KoShapeFactoryBase*> factories = priorityMap.values();
 
-    // Higher numbers are more specific, map is sorted by keys
+#ifndef NDEBUG
+    kDebug(30006) << "Supported factories for=" << p;
+    foreach (KoShapeFactoryBase *f, factories)
+        kDebug(30006) << f->id() << f->name();
+#endif
+
+    // Loop through all shape factories. If any of them supports this
+    // element, then we let the factory create a shape from it. This
+    // may fail because the element itself is too generic to draw any
+    // real conclusions from it - we actually have to try to load it.
+    // An example of this is the draw:image element which have
+    // potentially hundreds of different image formats to support,
+    // including vector formats.
+    //
+    // If it succeeds, then we use this shape, if it does fail, then
+    // just try the next.
+    //
+    // Higher numbers are more specific, map is sorted by keys.
     for (int i = factories.size() - 1; i >= 0; --i) {
-
-        KoShapeFactory * factory = factories[i];
+        KoShapeFactoryBase * factory = factories[i];
         if (factory->supports(element)) {
-
-            KoShape * shape = factory->createDefaultShapeAndInit(context.dataCenterMap());
+            KoShape *shape = factory->createDefaultShape(context.documentResourceManager());
 
             if (shape->shapeId().isEmpty())
                 shape->setShapeId(factory->id());
@@ -194,8 +242,18 @@ KoShape * KoShapeRegistry::createShapeInternal(const KoXmlElement & fullElement,
             bool loaded = shape->loadOdf(fullElement, context);
             context.odfLoadingContext().styleStack().restore();
 
-            if (loaded)
+            if (loaded) {
+                KoXmlElement text = KoXml::namedItemNS(fullElement, KoXmlNS::text, "p");
+                if (!text.isNull()) {
+                    KoTextOnShapeContainer *tos = new KoTextOnShapeContainer(shape,
+                            context.documentResourceManager());
+                    if (tos->loadOdf(fullElement, context)) {
+                        return tos;
+                    }
+                    delete tos;
+                }
                 return shape;
+            }
 
             // Maybe a shape with a lower priority can load our
             // element, but this attempt has failed.
@@ -204,7 +262,6 @@ KoShape * KoShapeRegistry::createShapeInternal(const KoXmlElement & fullElement,
     }
 
     return 0;
-
 }
 
-#include "KoShapeRegistry.moc"
+#include <KoShapeRegistry.moc>
