@@ -1,6 +1,6 @@
 /* This file is part of the KDE project
  *
- * Copyright (c) 2005-2009 Boudewijn Rempt <boud@valdyas.org>
+ * Copyright (c) 2005-2010 Boudewijn Rempt <boud@valdyas.org>
  * Copyright (C) 2006-2008 Thomas Zander <zander@kde.org>
  * Copyright (C) 2006 Thorsten Zachmann <zachmann@kde.org>
  * Copyright (C) 2008 Jan Hambrecht <jaham@gmx.net>
@@ -32,7 +32,7 @@
 #include "tools/KoCreateShapesTool.h"
 #include "tools/KoPathToolFactory.h"
 #include "KoCanvasController.h"
-#include "KoCanvasController_p.h"
+#include "KoCanvasControllerWidget.h"
 #include "KoShape.h"
 #include "KoShapeLayer.h"
 #include "KoShapeRegistry.h"
@@ -56,6 +56,7 @@
 #include <QKeyEvent>
 #include <QGridLayout>
 #include <QDockWidget>
+#include <QGraphicsWidget>
 #include <QStringList>
 #include <QAbstractButton>
 #include <QApplication>
@@ -242,11 +243,16 @@ void KoToolManager::Private::switchTool(KoToolBase *tool, bool temporary)
     // emit a empty status text to clear status text from last active tool
     emit q->changedStatusText(QString());
 
-    // we expect the tool to emit a cursor on activation.  This is for quick-fail :)
-    canvasData->canvas->canvas()->canvasWidget()->setCursor(Qt::ForbiddenCursor);
+    // we expect the tool to emit a cursor on activation.
+    updateCursor(Qt::ForbiddenCursor);
+
     foreach(KAction *action, canvasData->activeTool->actions()) {
         action->setEnabled(true);
-        canvasData->canvas->addAction(action);
+        // XXX: how to handle actions for non-qwidget-based canvases?
+        KoCanvasControllerWidget *canvasControllerWidget = dynamic_cast<KoCanvasControllerWidget*>(canvasData->canvas);
+        if (canvasControllerWidget) {
+            canvasControllerWidget->addAction(action);
+        }
     }
 
     postSwitchTool(temporary);
@@ -361,7 +367,10 @@ void KoToolManager::Private::postSwitchTool(bool temporary)
         action->setEnabled(true);
     }
 
-    canvasData->canvas->setToolOptionWidgets(optionWidgetMap);
+    KoCanvasControllerWidget *canvasControllerWidget = dynamic_cast<KoCanvasControllerWidget*>(canvasData->canvas);
+    if (canvasControllerWidget) {
+        canvasControllerWidget->setToolOptionWidgets(optionWidgetMap);
+    }
     emit q->changedTool(canvasData->canvas, uniqueToolIds.value(canvasData->activeTool));
 }
 
@@ -400,9 +409,15 @@ void KoToolManager::Private::detachCanvas(KoCanvasController *controller)
             // activate the found canvas controller
             canvasData = canvasses.value(newCanvas).first();
             inputDevice = canvasData->inputDevice;
-            canvasData->canvas->priv()->activate();
+            KoCanvasControllerWidget *canvasControllerWidget = dynamic_cast<KoCanvasControllerWidget*>(canvasData->canvas);
+            if (canvasControllerWidget) {
+                canvasControllerWidget->activate();
+            }
         } else {
-            canvasData->canvas->setToolOptionWidgets(QMap<QString, QWidget *>());
+            KoCanvasControllerWidget *canvasControllerWidget = dynamic_cast<KoCanvasControllerWidget*>(canvasData->canvas);
+            if (canvasControllerWidget) {
+                canvasControllerWidget->setToolOptionWidgets(QMap<QString, QWidget *>());
+            }
             // as a last resort just set a blank one
             canvasData = 0;
             // and stop the event filter
@@ -470,7 +485,10 @@ void KoToolManager::Private::attachCanvas(KoCanvasController *controller)
             SIGNAL(currentLayerChanged(const KoShapeLayer*)),
             q, SLOT(currentLayerChanged(const KoShapeLayer*)));
 
-    canvasData->canvas->priv()->activate();
+    KoCanvasControllerWidget *canvasControllerWidget = dynamic_cast<KoCanvasControllerWidget*>(canvasData->canvas);
+    if (canvasControllerWidget) {
+        canvasControllerWidget->activate();
+    }
 
     emit q->changedCanvas(canvasData ? canvasData->canvas->canvas() : 0);
 }
@@ -478,13 +496,24 @@ void KoToolManager::Private::attachCanvas(KoCanvasController *controller)
 void KoToolManager::Private::movedFocus(QWidget *from, QWidget *to)
 {
     Q_UNUSED(from);
-    if (to == 0 || (canvasData && to == canvasData->canvas))
+    // XXX: Focus handling for non-qwidget based canvases!
+    if (!canvasData) {
         return;
+    }
+
+    KoCanvasControllerWidget *canvasControllerWidget = dynamic_cast<KoCanvasControllerWidget*>(canvasData->canvas);
+    if (!canvasControllerWidget) {
+        return;
+    }
+
+    if (to == 0 || to == canvasControllerWidget) {
+        return;
+    }
 
     KoCanvasController *newCanvas = 0;
     // if the 'to' is one of our canvasses, or one of its children, then switch.
     foreach(KoCanvasController* canvas, canvasses.keys()) {
-        if (canvas == to || canvas->canvas()->canvasWidget() == to) {
+        if (canvasControllerWidget == to || canvas->canvas()->canvasWidget() == to) {
             newCanvas = canvas;
             break;
         }
@@ -500,7 +529,7 @@ void KoToolManager::Private::movedFocus(QWidget *from, QWidget *to)
     foreach(CanvasData *data, canvasses.value(newCanvas)) {
         if (data->inputDevice == inputDevice) {
             if (canvasData) { // deactivate the old one.
-                canvasData->canvas->canvas()->canvasWidget()->setCursor(Qt::ArrowCursor);
+                updateCursor(Qt::ArrowCursor);
                 if (canvasData->activeTool) {
                     canvasData->activeTool->deactivate();
                     KoToolProxy *proxy = proxies.value(canvasData->canvas->canvas());
@@ -510,8 +539,11 @@ void KoToolManager::Private::movedFocus(QWidget *from, QWidget *to)
             }
 
             canvasData = data;
-            canvasData->canvas->canvas()->canvasWidget()->setCursor(canvasData->activeTool->cursor());
-            canvasData->canvas->priv()->activate();
+            updateCursor(canvasData->activeTool->cursor());
+            KoCanvasControllerWidget *canvasControllerWidget = dynamic_cast<KoCanvasControllerWidget*>(canvasData->canvas);
+            if (canvasControllerWidget) {
+                canvasControllerWidget->activate();
+            }
             postSwitchTool(false);
             emit q->changedCanvas(canvasData ? canvasData->canvas->canvas() : 0);
             return;
@@ -520,7 +552,9 @@ void KoToolManager::Private::movedFocus(QWidget *from, QWidget *to)
     // no such inputDevice for this canvas...
     canvasData = canvasses.value(newCanvas).first();
     inputDevice = canvasData->inputDevice;
-    canvasData->canvas->priv()->activate();
+    if (canvasControllerWidget) {
+        canvasControllerWidget->activate();
+    }
     emit q->changedCanvas(canvasData ? canvasData->canvas->canvas() : 0);
 }
 
@@ -529,7 +563,7 @@ void KoToolManager::Private::updateCursor(const QCursor &cursor)
     Q_ASSERT(canvasData);
     Q_ASSERT(canvasData->canvas);
     Q_ASSERT(canvasData->canvas->canvas());
-    canvasData->canvas->canvas()->canvasWidget()->setCursor(cursor);
+    canvasData->canvas->canvas()->setCursor(cursor);
 }
 
 void KoToolManager::Private::switchBackRequested()
@@ -630,9 +664,12 @@ void KoToolManager::Private::switchInputDevice(const KoInputDevice &device)
                 switchTool(KoInteractionTool_ID, false);
             else {
                 postSwitchTool(false);
-                canvasData->canvas->canvas()->canvasWidget()->setCursor(canvasData->activeTool->cursor());
+                updateCursor(canvasData->activeTool->cursor());
+            }
+            KoCanvasControllerWidget *canvasControllerWidget = dynamic_cast<KoCanvasControllerWidget*>(canvasData->canvas);
+            if (canvasControllerWidget) {
+                canvasControllerWidget->activate();
             }
-            canvasData->canvas->priv()->activate();
             emit q->inputDeviceChanged(device);
             emit q->changedCanvas(canvasData ? canvasData->canvas->canvas() : 0);
             return;
@@ -650,7 +687,10 @@ void KoToolManager::Private::switchInputDevice(const KoInputDevice &device)
 
     q->switchToolRequested(oldTool);
     emit q->inputDeviceChanged(device);
-    canvasData->canvas->priv()->activate();
+    KoCanvasControllerWidget *canvasControllerWidget = dynamic_cast<KoCanvasControllerWidget*>(canvasData->canvas);
+    if (canvasControllerWidget) {
+        canvasControllerWidget->activate();
+    }
     emit q->changedCanvas(canvasData ? canvasData->canvas->canvas() : 0);
 }
 
@@ -768,16 +808,16 @@ void KoToolManager::addController(KoCanvasController *controller)
         return;
     d->setup();
     d->attachCanvas(controller);
-    connect(controller, SIGNAL(canvasRemoved(KoCanvasController*)), this, SLOT(detachCanvas(KoCanvasController*)));
-    connect(controller, SIGNAL(canvasSet(KoCanvasController*)), this, SLOT(attachCanvas(KoCanvasController*)));
+    connect(controller->proxyObject, SIGNAL(canvasRemoved(KoCanvasController*)), this, SLOT(detachCanvas(KoCanvasController*)));
+    connect(controller->proxyObject, SIGNAL(canvasSet(KoCanvasController*)), this, SLOT(attachCanvas(KoCanvasController*)));
 }
 
 void KoToolManager::removeCanvasController(KoCanvasController *controller)
 {
     Q_ASSERT(controller);
     d->detachCanvas(controller);
-    disconnect(controller, SIGNAL(canvasRemoved(KoCanvasController*)), this, SLOT(detachCanvas(KoCanvasController*)));
-    disconnect(controller, SIGNAL(canvasSet(KoCanvasController*)), this, SLOT(attachCanvas(KoCanvasController*)));
+    disconnect(controller->proxyObject, SIGNAL(canvasRemoved(KoCanvasController*)), this, SLOT(detachCanvas(KoCanvasController*)));
+    disconnect(controller->proxyObject, SIGNAL(canvasSet(KoCanvasController*)), this, SLOT(attachCanvas(KoCanvasController*)));
 }
 
 void KoToolManager::switchToolRequested(const QString & id)
