@@ -27,7 +27,7 @@
 #include <KoPathPointData.h>
 #include <KoViewConverter.h>
 #include <KoCanvasBase.h>
-#include <KoCanvasResourceProvider.h>
+#include <KoResourceManager.h>
 #include <QtGui/QPainter>
 
 KoPathToolSelection::KoPathToolSelection(KoPathTool * tool)
@@ -41,17 +41,17 @@ KoPathToolSelection::~KoPathToolSelection()
 
 void KoPathToolSelection::paint(QPainter &painter, const KoViewConverter &converter)
 {
-    int handleRadius = m_tool->canvas()->resourceProvider()->handleRadius();
+    int handleRadius = m_tool->canvas()->resourceManager()->handleRadius();
 
     PathShapePointMap::iterator it(m_shapePointMap.begin());
     for (; it != m_shapePointMap.end(); ++it) {
         painter.save();
 
-        painter.setMatrix(it.key()->absoluteTransformation(&converter) * painter.matrix());
+        painter.setTransform(it.key()->absoluteTransformation(&converter) * painter.transform());
         KoShape::applyConversion(painter, converter);
 
         foreach(KoPathPoint *p, it.value())
-        p->paint(painter, handleRadius, KoPathPoint::All);
+            p->paint(painter, handleRadius, KoPathPoint::All);
 
         painter.restore();
     }
@@ -61,7 +61,7 @@ void KoPathToolSelection::add(KoPathPoint * point, bool clear)
 {
     if( ! point )
         return;
-    
+
     bool allreadyIn = false;
     if (clear) {
         if (size() == 1 && m_selectedPoints.contains(point)) {
@@ -119,7 +119,7 @@ void KoPathToolSelection::selectPoints(const QRectF &rect, bool clearSelection)
         if (parameterShape && parameterShape->isParametricShape())
             continue;
         foreach(KoPathPoint* point, shape->pointsAt(shape->documentToShape(rect)))
-        add(point, false);
+            add(point, false);
     }
     blockSignals(false);
     emit selectionChanged();
@@ -165,8 +165,8 @@ QList<KoPathPointData> KoPathToolSelection::selectedSegmentsData() const
     KoPathPointData last(0, KoPathPointIndex(-1, -1));
     KoPathPointData lastSubpathStart(0, KoPathPointIndex(-1, -1));
 
-    QList<KoPathPointData>::const_iterator it(pd.begin());
-    for (; it != pd.end(); ++it) {
+    QList<KoPathPointData>::const_iterator it(pd.constBegin());
+    for (; it != pd.constEnd(); ++it) {
         if (it->pointIndex.second == 0)
             lastSubpathStart = *it;
 
@@ -209,7 +209,7 @@ void KoPathToolSelection::repaint()
 void KoPathToolSelection::update()
 {
     bool selectionHasChanged = false;
-    
+
     PathShapePointMap::iterator it(m_shapePointMap.begin());
     while (it != m_shapePointMap.end()) {
         KoParameterShape *parameterShape = dynamic_cast<KoParameterShape*>(it.key());
@@ -218,7 +218,7 @@ void KoPathToolSelection::update()
             QSet<KoPathPoint *>::iterator pointIt(it.value().begin());
             for (; pointIt != it.value().end(); ++pointIt) {
                 m_selectedPoints.remove( *pointIt );
-            }    
+            }
             it = m_shapePointMap.erase(it);
             selectionHasChanged = true;
         } else {
@@ -235,7 +235,7 @@ void KoPathToolSelection::update()
             ++it;
         }
     }
-    
+
     if (selectionHasChanged)
         emit selectionChanged();
 }
@@ -245,4 +245,4 @@ bool KoPathToolSelection::hasSelection()
     return !m_selectedPoints.isEmpty();
 }
 
-#include "KoPathToolSelection.moc"
+#include <KoPathToolSelection.moc>
