@@ -76,6 +76,8 @@
 
 #include <qxml.h>
 #include <qdom.h>
+#include <QXmlStreamReader>
+#include <QXmlStreamEntityResolver>
 
 #include <QBuffer>
 #include <QByteArray>
@@ -110,6 +112,29 @@
 // this is used to quickly get namespaced attribute(s)
 typedef QPair<QString, QString> KoXmlStringPair;
 
+class KoQName {
+public:
+    QString nsURI;
+    QString name;
+
+    explicit KoQName(const QString& nsURI_, const QString& name_)
+        : nsURI(nsURI_), name(name_) {}
+    bool operator==(const KoQName& qname) const {
+        // local name is more likely to differ, so compare that first
+        return name == qname.name && nsURI == qname.nsURI;
+    }
+};
+
+uint qHash(const KoQName& qname)
+{
+    // possibly add a faster hash function that only includes some trailing
+    // part of the nsURI
+
+    // in case of doubt, use this:
+    // return qHash(qname.nsURI)^qHash(qname.name);
+    return qHash(qname.nsURI)^qHash(qname.name);
+}
+
 // this simplistic hash is rather fast-and-furious. it works because
 // likely there is very few namespaced attributes per element
 static inline uint qHash(const KoXmlStringPair &p)
@@ -181,7 +206,7 @@ static QString fixNamespace(const QString &nsURI)
 //
 // ==================================================================
 
-// 24 bytes on most system
+// 12 bytes on most system 32 bit systems, 16 bytes on 64 bit systems
 class KoXmlPackedItem
 {
 public:
@@ -194,8 +219,7 @@ quint32 childStart: 28;
 unsigned depth: 28;
 #endif
 
-    unsigned nameIndex;
-    unsigned nsURIIndex;
+    unsigned qnameIndex;
     QString value;
 
     // it is important NOT to have a copy constructor, so that growth is optimal
@@ -215,8 +239,7 @@ static QDataStream& operator<<(QDataStream& s, const KoXmlPackedItem& item)
     s << flag;
     s << (quint8) item.type;
     s << item.childStart;
-    s << item.nameIndex;
-    s << item.nsURIIndex;
+    s << item.qnameIndex;
     s << item.value;
 
     return s;
@@ -232,8 +255,7 @@ static QDataStream& operator>>(QDataStream& s, KoXmlPackedItem& item)
     s >> flag;
     s >> type;
     s >> child;
-    s >> item.nameIndex;
-    s >> item.nsURIIndex;
+    s >> item.qnameIndex;
     s >> value;
 
     item.attr = (flag != 0);
@@ -717,24 +739,23 @@ public:
     QVector<KoXmlPackedItem> items;
 #endif
 
-    QStringList stringList;
+    QList<KoQName> qnameList;
     QString docType;
 
 private:
-    QHash<QString, unsigned> stringHash;
+    QHash<KoQName, unsigned> qnameHash;
 
-    unsigned cacheString(const QString& str) {
-        if (str.isEmpty())
-            return 0;
+    unsigned cacheQName(const QString& name, const QString& nsURI) {
+        KoQName qname(nsURI, name);
 
-        const unsigned& ii = stringHash[str];
-        if (ii > 0)
+        const unsigned ii = qnameHash.value(qname, (unsigned)-1);
+        if (ii != (unsigned)-1)
             return ii;
 
         // not yet declared, so we add it
-        unsigned i = stringList.count();
-        stringList.append(str);
-        stringHash.insert(str, i);
+        unsigned i = qnameList.count();
+        qnameList.append(qname);
+        qnameHash.insert(qname, i);
 
         return i;
     }
@@ -761,7 +782,7 @@ private:
 #ifdef KOXML_COMPACT
 public:
     const KoXmlPackedItem& itemAt(unsigned depth, unsigned index) {
-        KoXmlPackedGroup& group = groups[depth];
+        const KoXmlPackedGroup& group = groups[depth];
         return group[index];
     }
 
@@ -775,7 +796,7 @@ public:
           Function clear, newItem, addElement, addAttribute, addText,
           addCData, addProcessing are all related. These are all necessary
           for stateful manipulation of the document. See also the calls
-          to these function from KoXmlHandler.
+          to these function from parseDocument().
 
           The state itself is defined by the member variables
           currentDepth and the groups (see above).
@@ -801,8 +822,7 @@ public:
         // a constructor for KoXmlPackedItem
         item.attr = false;
         item.type = KoXmlNode::NullNode;
-        item.nameIndex = 0;
-        item.nsURIIndex = 0;
+        item.qnameIndex = 0;
         item.childStart = itemCount(depth + 1);
         item.value.clear();
 
@@ -811,16 +831,13 @@ public:
 
     void clear() {
         currentDepth = 0;
-        stringHash.clear();
-        stringList.clear();
+        qnameHash.clear();
+        qnameList.clear();
         valueHash.clear();
         valueList.clear();
         groups.clear();
         docType.clear();
 
-        // reserve index #0
-        cacheString(QString());
-
         // first node is root
         KoXmlPackedItem& rootItem = newItem(0);
         rootItem.type = KoXmlNode::DocumentNode;
@@ -828,7 +845,7 @@ public:
 
     void finish() {
         // won't be needed anymore
-        stringHash.clear();
+        qnameHash.clear();
         valueHash.clear();
         valueList.clear();
 
@@ -843,8 +860,7 @@ public:
     void addElement(const QString& name, const QString& nsURI) {
         KoXmlPackedItem& item = newItem(currentDepth + 1);
         item.type = KoXmlNode::ElementNode;
-        item.nameIndex = cacheString(name);
-        item.nsURIIndex = cacheString(nsURI);
+        item.qnameIndex = cacheQName(name, nsURI);
 
         currentDepth++;
     }
@@ -860,8 +876,7 @@ public:
     void addAttribute(const QString& name, const QString& nsURI, const QString& value) {
         KoXmlPackedItem& item = newItem(currentDepth + 1);
         item.attr = true;
-        item.nameIndex = cacheString(name);
-        item.nsURIIndex = cacheString(nsURI);
+        item.qnameIndex = cacheQName(name, nsURI);
         //item.value = cacheValue( value );
         item.value = value;
     }
@@ -1010,237 +1025,110 @@ public:
 
 };
 
-// ==================================================================
-//
-//         KoXmlHandler
-//
-// ==================================================================
+namespace {
 
-class KoXmlHandler : public QXmlDefaultHandler
-{
-public:
-    KoXmlHandler(KoXmlPackedDocument* doc);
+    class ParseError {
+    public:
+        QString errorMsg;
+        int errorLine;
+        int errorColumn;
+        bool error;
 
-    // content handler
-    bool startDocument();
-    bool endDocument();
+        ParseError() :errorLine(-1), errorColumn(-1), error(false) {}
+    };
 
-    bool startElement(const QString& nsURI, const QString& localName,
-                      const QString& qName, const QXmlAttributes& atts);
-    bool endElement(const QString& nsURI, const QString& localName,
-                    const QString& qName);
+    void parseElement(QXmlStreamReader &xml, KoXmlPackedDocument &doc);
 
-    bool characters(const QString& ch);
-    bool processingInstruction(const QString& target, const QString& data);
-    bool skippedEntity(const QString& name);
-
-    // lexical handler
-    bool startCDATA();
-    bool endCDATA();
-    bool startEntity(const QString &);
-    bool endEntity(const QString &);
-    bool startDTD(const QString& name, const QString& publicId,
-                  const QString& systemId);
-    bool comment(const QString& ch);
-
-    // decl handler
-    bool externalEntityDecl(const QString &name, const QString &publicId,
-                            const QString &systemId) ;
-
-    // DTD handler
-    bool notationDecl(const QString & name, const QString & publicId,
-                      const QString & systemId);
-    bool unparsedEntityDecl(const QString &name, const QString &publicId,
-                            const QString &systemId, const QString &notationName) ;
-
-    // error handler
-    bool fatalError(const QXmlParseException& exception);
-
-    QString errorMsg;
-    int errorLine;
-    int errorColumn;
-
-private:
-    KoXmlPackedDocument* document;
-    bool processNamespace;
-    QString entityName;
-    bool cdata;
-};
-
-
-KoXmlHandler::KoXmlHandler(KoXmlPackedDocument* doc) : QXmlDefaultHandler(),
-        errorLine(0), errorColumn(0), document(doc),
-        processNamespace(doc->processNamespace), cdata(false)
-{
-}
-
-bool KoXmlHandler::startDocument()
-{
-    // just for sanity
-    cdata = false;
-    entityName.clear();
-
-    errorMsg.clear();
-    errorLine = 0;
-    errorColumn = 0;
-
-    document->clear();
-    return true;
-}
-
-bool KoXmlHandler::endDocument()
-{
-    document->finish();
-    return true;
-}
-
-bool KoXmlHandler::startDTD(const QString& name, const QString& publicId,
-                            const QString& systemId)
-{
-    Q_UNUSED(publicId);
-    Q_UNUSED(systemId);
-
-    document->addDTD(name);
-
-    return true;
-}
-
-bool KoXmlHandler::startElement(const QString& nsURI, const QString& localName,
-                                const QString& name, const QXmlAttributes& atts)
-{
-    Q_UNUSED(localName);
-
-    document->addElement(name, fixNamespace(nsURI));
-
-    // add all attributes
-    for (int c = 0; c < atts.length(); c++)
-        document->addAttribute(atts.qName(c), atts.uri(c), atts.value(c));
-
-    return true;
-}
-
-bool KoXmlHandler::endElement(const QString& nsURI, const QString& localName,
-                              const QString& qName)
-{
-    Q_UNUSED(nsURI);
-    Q_UNUSED(localName);
-    Q_UNUSED(qName);
-
-    document->closeElement();
-
-    return true;
-}
-
-bool KoXmlHandler::characters(const QString& str)
-{
-    // are we inside entity ?
-    if (!entityName.isEmpty()) {
-        // we do not handle entity but need to keep track of it
-        // because we want to skip it alltogether
-        return true;
+    // parse one element as if this were a standalone xml document
+    ParseError parseDocument(QXmlStreamReader &xml, KoXmlPackedDocument &doc) {
+        doc.clear();
+        ParseError error;
+        xml.readNext();
+        while (!xml.atEnd() && xml.tokenType() != QXmlStreamReader::EndDocument) {
+            switch (xml.tokenType()) {
+            case QXmlStreamReader::StartElement:
+                parseElement(xml, doc);
+                break;
+            case QXmlStreamReader::DTD:
+                doc.addDTD(xml.dtdName().toString());
+                break;
+            case QXmlStreamReader::StartDocument:
+                if (!xml.documentEncoding().isEmpty() || !xml.documentVersion().isEmpty()) {
+                    doc.addProcessingInstruction();
+                }
+                break;
+            case QXmlStreamReader::ProcessingInstruction:
+                doc.addProcessingInstruction();
+                break;
+            default:
+                break;
+            }
+            xml.readNext();
+        }
+        if (xml.hasError()) {
+            error.error = true;
+            error.errorMsg = xml.errorString();
+            error.errorColumn = xml.columnNumber();
+            error.errorLine = xml.lineNumber();
+        } else {
+            doc.finish();
+        }
+        return error;
     }
 
-    // add a new text or CDATA
-    if (cdata)
-        document->addCData(str);
-    else
-        document->addText(str);
+    void parseElementContents(QXmlStreamReader &xml, KoXmlPackedDocument &doc)
+    {
+        xml.readNext();
+        QString ws;
+        bool sawElement = false;
+        while (!xml.atEnd()) {
+            switch (xml.tokenType()) {
+            case QXmlStreamReader::EndElement:
+                // if an element contains only whitespace, put it in the dom
+                if (!ws.isEmpty() && !sawElement) {
+                    doc.addText(ws);
+                }
+                return;
+            case QXmlStreamReader::StartElement:
+                sawElement = true;
+                parseElement(xml, doc);
+                break;
+            case QXmlStreamReader::Characters:
+                if (xml.isCDATA()) {
+                    doc.addCData(xml.text().toString());
+                } else if (!xml.isWhitespace()) {
+                    doc.addText(xml.text().toString());
+                } else if (!sawElement) {
+                    ws += xml.text();
+                }
+                break;
+            case QXmlStreamReader::ProcessingInstruction:
+                doc.addProcessingInstruction();
+                break;
+            default:
+                break;
+            }
+            xml.readNext();
+        }
+    }
 
-    return true;
-}
+    void parseElement(QXmlStreamReader &xml, KoXmlPackedDocument &doc) {
+        // reader.tokenType() is now QXmlStreamReader::StartElement
+        doc.addElement(xml.qualifiedName().toString(),
+                       fixNamespace(xml.namespaceUri().toString()));
+        QXmlStreamAttributes attr = xml.attributes();
+        QXmlStreamAttributes::const_iterator a = attr.constBegin();
+        while (a != attr.constEnd()) {
+            doc.addAttribute(a->qualifiedName().toString(),
+                             a->namespaceUri().toString(),
+                             a->value().toString());
+            ++a;
+        }
+        parseElementContents(xml, doc);
+        // reader.tokenType() is now QXmlStreamReader::EndElement
+        doc.closeElement();
+    }
 
-bool KoXmlHandler::processingInstruction(const QString& target,
-        const QString& data)
-{
-    Q_UNUSED(target);
-    Q_UNUSED(data);
-
-    document->addProcessingInstruction();
-
-    return true;
-}
-
-bool KoXmlHandler::skippedEntity(const QString& name)
-{
-    Q_UNUSED(name);
-
-    // we skip entity
-    return true;
-}
-
-bool KoXmlHandler::startCDATA()
-{
-    cdata = true;
-    return true;
-}
-
-bool KoXmlHandler::endCDATA()
-{
-    cdata = false;
-    return true;
-}
-
-bool KoXmlHandler::startEntity(const QString& name)
-{
-    entityName = name;
-    return true;
-}
-
-bool KoXmlHandler::endEntity(const QString& name)
-{
-    Q_UNUSED(name);
-    entityName.clear();
-    return true;
-}
-
-bool KoXmlHandler::comment(const QString& comment)
-{
-    Q_UNUSED(comment);
-
-    // we skip comment
-    return true;
-}
-
-bool KoXmlHandler::unparsedEntityDecl(const QString &name,
-                                      const QString &publicId, const QString &systemId, const QString &notationName)
-{
-    Q_UNUSED(name);
-    Q_UNUSED(publicId);
-    Q_UNUSED(systemId);
-    Q_UNUSED(notationName);
-
-    // we skip entity
-    return true;
-}
-
-bool KoXmlHandler::externalEntityDecl(const QString &name,
-                                      const QString &publicId, const QString &systemId)
-{
-    Q_UNUSED(name);
-    Q_UNUSED(publicId);
-    Q_UNUSED(systemId);
-
-    // we skip entity
-    return true;
-}
-
-bool KoXmlHandler::notationDecl(const QString & name,
-                                const QString & publicId, const QString & systemId)
-{
-    Q_UNUSED(name);
-    Q_UNUSED(publicId);
-    Q_UNUSED(systemId);
-
-    // we skip notation node
-    return true;
-}
-
-bool KoXmlHandler::fatalError(const QXmlParseException& exception)
-{
-    errorMsg = exception.message();
-    errorLine =  exception.lineNumber();
-    errorColumn =  exception.columnNumber();
-    return QXmlDefaultHandler::fatalError(exception);
 }
 
 
@@ -1313,9 +1201,7 @@ public:
     unsigned long nodeIndex;
 
     // for document node
-    bool setContent(QXmlInputSource* source, QXmlReader* reader,
-                    QString* errorMsg = 0, int* errorLine = 0, int* errorColumn = 0);
-    bool setContent(QXmlInputSource* source, bool namespaceProcessing,
+    bool setContent(QXmlStreamReader *reader,
                     QString* errorMsg = 0, int* errorLine = 0, int* errorColumn = 0);
 
     // used when doing on-demand (re)parse
@@ -1334,7 +1220,6 @@ private:
     QHash<QString, QString> attr;
     QHash<KoXmlStringPair, QString> attrNS;
     QString textData;
-    friend class KoXmlHandler;
     friend class KoXmlElement;
 };
 
@@ -1520,18 +1405,7 @@ void KoXmlNodeData::setData(const QString& d)
     textData = d;
 }
 
-bool KoXmlNodeData::setContent(QXmlInputSource* source, bool namespaceProcessing,
-                               QString* errorMsg, int* errorLine, int* errorColumn)
-{
-    QXmlSimpleReader reader;
-    reader.setFeature(QLatin1String("http://xml.org/sax/features/namespaces"), namespaceProcessing);
-    reader.setFeature(QLatin1String("http://xml.org/sax/features/namespace-prefixes"), !namespaceProcessing);
-    reader.setFeature(QLatin1String("http://trolltech.com/xml/features/report-whitespace-only-CharData"), false);
-    return setContent(source, &reader, errorMsg, errorLine, errorColumn);
-}
-
-bool KoXmlNodeData::setContent(QXmlInputSource* source,
-                               QXmlReader* reader, QString* errorMsg, int* errorLine, int* errorColumn)
+bool KoXmlNodeData::setContent(QXmlStreamReader* reader, QString* errorMsg, int* errorLine, int* errorColumn)
 {
     if (nodeType != KoXmlNode::DocumentNode)
         return false;
@@ -1540,34 +1414,21 @@ bool KoXmlNodeData::setContent(QXmlInputSource* source,
     nodeType = KoXmlNode::DocumentNode;
 
     // sanity checks
-    if (!source) return false;
     if (!reader) return false;
 
     delete packedDoc;
     packedDoc = new KoXmlPackedDocument;
-    packedDoc->processNamespace = false;
+    packedDoc->processNamespace = reader->namespaceProcessing();
 
-    packedDoc->processNamespace =
-        reader->feature("http://xml.org/sax/features/namespaces") &&
-        !reader->feature("http://xml.org/sax/features/namespace-prefixes");
-
-    KoXmlHandler handler(packedDoc);
-    reader->setContentHandler(&handler);
-    reader->setErrorHandler(&handler);
-    reader->setLexicalHandler(&handler);
-    reader->setDeclHandler(&handler);
-    reader->setDTDHandler(&handler);
-
-    bool result = reader->parse(source);
-    if (!result) {
+    ParseError error = parseDocument(*reader, *packedDoc);
+    if (error.error) {
         // parsing error has occurred
-        if (errorMsg) *errorMsg = handler.errorMsg;
-        if (errorLine) *errorLine = handler.errorLine;
-        if (errorColumn)  *errorColumn = handler.errorColumn;
+        if (errorMsg) *errorMsg = error.errorMsg;
+        if (errorLine) *errorLine = error.errorLine;
+        if (errorColumn)  *errorColumn = error.errorColumn;
         return false;
     }
 
-
     // initially load
     loadChildren();
 
@@ -1607,8 +1468,7 @@ void KoXmlNodeData::loadChildren(int depth)
 
         // attribute belongs to this node
         if (item.attr) {
-            QString name = packedDoc->stringList[item.nameIndex];
-            QString nsURI = fixNamespace(packedDoc->stringList[item.nsURIIndex]);
+            KoQName qname = packedDoc->qnameList[item.qnameIndex];
             QString value = item.value;
 
             QString prefix;
@@ -1616,31 +1476,30 @@ void KoXmlNodeData::loadChildren(int depth)
             QString qName; // with prefix
             QString localName;  // without prefix, i.e. local name
 
-            localName = qName = name;
+            localName = qName = qname.name;
             int i = qName.indexOf(':');
             if (i != -1) prefix = qName.left(i);
             if (i != -1) localName = qName.mid(i + 1);
 
             if (packedDoc->processNamespace) {
-                setAttributeNS(nsURI, qName, value);
+                setAttributeNS(qname.nsURI, qName, value);
                 setAttribute(localName, value);
             } else
                 setAttribute(qName, value);
         } else {
-            QString name = packedDoc->stringList[item.nameIndex];
-            QString nsURI = fixNamespace(packedDoc->stringList[item.nsURIIndex]);
+            KoQName qname = packedDoc->qnameList[item.qnameIndex];
             QString value = item.value;
 
-            QString nodeName = name;
+            QString nodeName = qname.name;
             QString localName;
             QString prefix;
 
             if (packedDoc->processNamespace) {
-                localName = name;
-                int di = name.indexOf(':');
+                localName = qname.name;
+                int di = qname.name.indexOf(':');
                 if (di != -1) {
-                    localName = name.mid(di + 1);
-                    prefix = name.left(di);
+                    localName = qname.name.mid(di + 1);
+                    prefix = qname.name.left(di);
                 }
                 nodeName = localName;
             }
@@ -1654,7 +1513,7 @@ void KoXmlNodeData::loadChildren(int depth)
             dat->tagName = nodeName;
             dat->localName = localName;
             dat->prefix = prefix;
-            dat->namespaceURI = nsURI;
+            dat->namespaceURI = qname.nsURI;
             dat->count = 1;
             dat->parent = this;
             dat->prev = lastDat;
@@ -1839,13 +1698,13 @@ static QDomNode itemAsQDomNode(QDomDocument ownerDoc, KoXmlPackedDocument* packe
     if (self.type == KoXmlNode::ElementNode) {
         QDomElement element;
 
-        QString name = packedDoc->stringList[self.nameIndex];
-        QString nsURI = fixNamespace(packedDoc->stringList[self.nsURIIndex]);
+        KoQName qname = packedDoc->qnameList[self.qnameIndex];
+        qname.nsURI = fixNamespace(qname.nsURI);
 
         if (packedDoc->processNamespace)
-            element = ownerDoc.createElementNS(nsURI, name);
+            element = ownerDoc.createElementNS(qname.nsURI, qname.name);
         else
-            element = ownerDoc.createElement(name);
+            element = ownerDoc.createElement(qname.name);
 
         // check all subnodes for attributes
         for (unsigned i = self.childStart; i < childStop; i++) {
@@ -1855,8 +1714,8 @@ static QDomNode itemAsQDomNode(QDomDocument ownerDoc, KoXmlPackedDocument* packe
 
             // attribute belongs to this node
             if (item.attr) {
-                QString name = packedDoc->stringList[item.nameIndex];
-                QString nsURI = fixNamespace(packedDoc->stringList[item.nsURIIndex]);
+                KoQName qname = packedDoc->qnameList[item.qnameIndex];
+                qname.nsURI = fixNamespace(qname.nsURI );
                 QString value = item.value;
 
                 QString prefix;
@@ -1864,16 +1723,16 @@ static QDomNode itemAsQDomNode(QDomDocument ownerDoc, KoXmlPackedDocument* packe
                 QString qName; // with prefix
                 QString localName;  // without prefix, i.e. local name
 
-                localName = qName = name;
+                localName = qName = qname.name;
                 int i = qName.indexOf(':');
                 if (i != -1) prefix = qName.left(i);
                 if (i != -1) localName = qName.mid(i + 1);
 
                 if (packedDoc->processNamespace) {
-                    element.setAttributeNS(nsURI, qName, value);
+                    element.setAttributeNS(qname.nsURI, qName, value);
                     element.setAttribute(localName, value);
                 } else
-                    element.setAttribute(name, value);
+                    element.setAttribute(qname.name, value);
             } else {
                 // add it recursively
                 QDomNode childNode = itemAsQDomNode(ownerDoc, packedDoc, nodeDepth + 1, i);
@@ -2612,7 +2471,18 @@ void KoXmlDocument::clear()
     d->emptyDocument = false;
 }
 
-bool KoXmlDocument::setContent(QXmlInputSource *source, QXmlReader *reader,
+namespace {
+    /* Use an entity resolver that ignores undefined entities and simply
+       returns an empty string for them.
+       */
+    class DumbEntityResolver : public QXmlStreamEntityResolver {
+    public:
+        QString resolveUndeclaredEntity ( const QString &) { return ""; }
+    };
+
+}
+
+bool KoXmlDocument::setContent(QXmlStreamReader *reader,
                                QString* errorMsg, int* errorLine, int* errorColumn)
 {
     if (d->nodeType != KoXmlNode::DocumentNode) {
@@ -2622,7 +2492,7 @@ bool KoXmlDocument::setContent(QXmlInputSource *source, QXmlReader *reader,
     }
 
     dt = KoXmlDocumentType();
-    bool result = d->setContent(source, reader, errorMsg, errorLine, errorColumn);
+    bool result = d->setContent(reader, errorMsg, errorLine, errorColumn);
     if (result && !isNull()) {
         dt.d->nodeType = KoXmlNode::DocumentTypeNode;
         dt.d->tagName = d->packedDoc->docType;
@@ -2648,18 +2518,14 @@ bool KoXmlDocument::setContent(QIODevice* device, bool namespaceProcessing,
         d->nodeType = KoXmlNode::DocumentNode;
     }
 
-    QXmlSimpleReader reader;
-    reader.setFeature("http://xml.org/sax/features/namespaces", namespaceProcessing);
-    reader.setFeature("http://xml.org/sax/features/namespace-prefixes", !namespaceProcessing);
-    reader.setFeature(QLatin1String("http://trolltech.com/xml/features/report-whitespace-only-CharData"), false);
-
-    // FIXME this hack is apparently private
-    //reader.setUndefEntityInAttrHack(true);
-
-    QXmlInputSource source(device);
+    device->open(QIODevice::ReadOnly);
+    QXmlStreamReader reader(device);
+    reader.setNamespaceProcessing(namespaceProcessing);
+    DumbEntityResolver entityResolver;
+    reader.setEntityResolver(&entityResolver);
 
     dt = KoXmlDocumentType();
-    bool result = d->setContent(&source, &reader, errorMsg, errorLine, errorColumn);
+    bool result = d->setContent(&reader, errorMsg, errorLine, errorColumn);
     {
         dt.d->nodeType = KoXmlNode::DocumentTypeNode;
         dt.d->tagName = d->packedDoc->docType;
@@ -2686,11 +2552,13 @@ bool KoXmlDocument::setContent(const QString& text, bool namespaceProcessing,
         d->nodeType = KoXmlNode::DocumentNode;
     }
 
-    QXmlInputSource source;
-    source.setData(text);
+    QXmlStreamReader reader(text);
+    reader.setNamespaceProcessing(namespaceProcessing);
+    DumbEntityResolver entityResolver;
+    reader.setEntityResolver(&entityResolver);
 
     dt = KoXmlDocumentType();
-    bool result = d->setContent(&source, namespaceProcessing, errorMsg, errorLine, errorColumn);
+    bool result = d->setContent(&reader, errorMsg, errorLine, errorColumn);
     if (result && !isNull()) {
         dt.d->nodeType = KoXmlNode::DocumentTypeNode;
         dt.d->tagName = d->packedDoc->docType;
@@ -2708,92 +2576,6 @@ bool KoXmlDocument::setContent(const QString& text,
 
 #endif
 
-// ==================================================================
-//
-//         KoXmlInputSource
-//
-// ==================================================================
-
-/*
-  This is the size of buffer every time we read a chunk of data from the device.
-
-  Note 1: maximum allocated space is thus 2*KOXML_BUFSIZE due to the
-  stringData (a QString instance).
-  TODO: use mmap to avoid double-buffering like this.
-
-  Note 2: a much larger buffer won't speed up significantly. This is because
-  the bottleneck is elsewhere, not here.
-
-*/
-
-#define KOXML_BUFSIZE 16*1024  // should be adequate
-
-KoXmlInputSource::KoXmlInputSource(QIODevice *dev): QXmlInputSource(),
-        device(dev)
-{
-    int mib = 106; // UTF-8
-    decoder = QTextCodec::codecForMib(mib)->makeDecoder();
-
-    stringLength = 0;
-    stringIndex = 0;
-    buffer = new char[KOXML_BUFSIZE];
-
-    reset();
-}
-
-KoXmlInputSource::~KoXmlInputSource()
-{
-    delete decoder;
-    delete [] buffer;
-}
-
-void KoXmlInputSource::setData(const QString& dat)
-{
-    Q_UNUSED(dat);
-}
-
-void KoXmlInputSource::setData(const QByteArray& dat)
-{
-    Q_UNUSED(dat);
-}
-
-void KoXmlInputSource::fetchData()
-{
-}
-
-QString KoXmlInputSource::data() const
-{
-    return QString();
-}
-
-QChar KoXmlInputSource::next()
-{
-    if (stringIndex >= stringLength) {
-        // read more data first
-        qint64 bytes = device->read(buffer, KOXML_BUFSIZE);
-        if (bytes == 0)
-            return EndOfDocument;
-
-        stringData = decoder->toUnicode(buffer, bytes);
-        stringLength = stringData.length();
-        stringIndex = 0;
-    }
-
-    return stringData[stringIndex++];
-}
-
-void KoXmlInputSource::reset()
-{
-    device->seek(0);
-}
-
-QString KoXmlInputSource::fromRawData(const QByteArray &data, bool beginning)
-{
-    Q_UNUSED(data);
-    Q_UNUSED(beginning);
-    return QString();
-}
-
 // ==================================================================
 //
 //         functions in KoXml namespace
@@ -2891,23 +2673,8 @@ bool KoXml::setDocument(KoXmlDocument& doc, QIODevice* device,
                         bool namespaceProcessing, QString* errorMsg, int* errorLine,
                         int* errorColumn)
 {
-    QXmlSimpleReader reader;
-    reader.setFeature(QLatin1String("http://xml.org/sax/features/namespaces"), namespaceProcessing);
-    reader.setFeature(QLatin1String("http://xml.org/sax/features/namespace-prefixes"), !namespaceProcessing);
-    reader.setFeature(QLatin1String("http://trolltech.com/xml/features/report-whitespace-only-CharData"), false);
-
-    KoXmlInputSource* source = new KoXmlInputSource(device);
-    bool result = doc.setContent(source, &reader, errorMsg, errorLine, errorColumn);
-    delete source;
+    QXmlStreamReader reader(device);
+    reader.setNamespaceProcessing(namespaceProcessing);
+    bool result = doc.setContent(&reader, errorMsg, errorLine, errorColumn);
     return result;
 }
-
-bool KoXml::setDocument(KoXmlDocument& doc, QIODevice* device,
-                        QXmlSimpleReader* reader, QString* errorMsg, int* errorLine, int* errorColumn)
-{
-    KoXmlInputSource* source = new KoXmlInputSource(device);
-    bool result = doc.setContent(source, reader, errorMsg, errorLine, errorColumn);
-    delete source;
-    return result;
-}
-
