@@ -22,6 +22,8 @@
 
 #include <KoToolFactoryBase.h>
 
+class FoCellTool;
+
 /**
  * The Factory, that creates a FoCellTool.
  */
@@ -32,6 +34,14 @@ public:
     ~FoCellToolFactory();
 
     KoToolBase* createTool(KoCanvasBase* canvas);
+
+    FoCellTool *cellTool();
+
+private:
+    /*!
+     * Contains the pointer to the first created CellTool.
+     */
+    FoCellTool *m_cellTool;
 };
 
 #endif // FOCELL_TOOL_FACTORY
